/*
 * ------------------------------------------------------------
 * Contains reader functions for TSPTW instances
 * ------------------------------------------------------------
 */

#ifndef TSPTW_INSTANCES_HPP_
#define TSPTW_INSTANCES_HPP_

#include <cassert>
#include "../constraints/disjunctive.hpp"
#include <ilcp/cpext.h>


/**
 * Read RBG TSPTW instance
 **/
void read_afg_input(IloEnv env, char* filename, ActivityArray &acts, int** &setup);



/**
 * Read DUMAS TSPTW instance
 **/
void read_dumas_input(IloEnv env, char* filename, ActivityArray &acts, int** &setup);


/**
 * Read weighted total tardiness instance
 **/
void read_wt_input(IloEnv env, char* filename, ActivityArray &acts, int** &setup);


/**
 * Read disjunctive instances
 **/
void read_disj_input(IloEnv env, char* filename, ActivityArray &acts, int** &setup);


/**
 * Read randomly generated instances
 **/
void read_random_input(IloEnv env, char* filename, ActivityArray &acts, int** &setup);


/**
 * Read weighted total tardiness instance with setup times
 **/
void read_wts_input(IloEnv env, char* filename, ActivityArray &acts, int** &setup);


/**
 * Read Sequential Ordering Problems
 **/
void read_sop_input(IloEnv env, char* filename, ActivityArray &acts, int** &setup, bool** &precedes);

/**
 * Read TSP with Pickup and Delivery instance
 **/
void read_tsppd_input(IloEnv env, char* filename, ActivityArray &acts, int** &setup, bool** &precedes);


/**
 * Extract instance name
 */
void extract_instance_name(char* _filename, char* name);



#endif /* TSPTW_INSTANCES_HPP_ */
