//#include <cstdio>
//#include <iostream>
//#include <ilcp/cpext.h>
//#include <vector>
//
//#define TIME_LIMIT 300
//#define OPT_TOL 1e-8
//
//using namespace std;
//
//
///**
// * Activity
// **/
//struct Activity {
//	int					id;					/**< unique activity identification */
//	int					release;			/**< original activity release time */
//	int 				deadline;			/**< original activity deadline */
//	int					processing;			/**< original activity processing time */
//	int                 duedate;           /**< activity due-date */
//	int                 weight;            /**< activity tardiness weight */
//
//	IloIntervalVar		ilo_start_time;		/**< model interval variable for start time */
//	IlcIntervalVar		start_time;			/**< interval variable for start time*/
//
//	/** Setters */
//	void set_processing(int p);
//	void set_duedate(int d);
//	void set_weight(int w);
//
//	Activity(IloEnv env, int _id, int _release, int _deadline, int _processing);
//	Activity(IloEnv env, int _id, int _release, int _deadline, int _processing, int _due_date, int _weight);
//	Activity(IloEnv env, IloIntervalVar _ilo_act, int _id, int _release, int _deadline, int _processing);
//};
//
//
///** Array of activities */
//typedef vector<Activity*> ActivityArray;
//
//
///**
// * Read sequential ordering Problem
// **/
//void read_sop_instance(
//		IloEnv env,
//		char* filename,
//		ActivityArray &acts,
//		int** &setup,
//		bool** &precedes)
//{
//	ifstream input(filename);
//	if( !input.is_open() ) {
//		cerr << "Error: could not open file " << filename << endl;
//		exit(1);
//	}
//
//	char str_in[256];
//	input >> str_in;
//	while( strcmp(str_in, "EDGE_WEIGHT_SECTION") != 0 && strcmp(str_in, "edge_weight_section") != 0 ) {
//		input >> str_in;
//	}
//
//	int num_acts;
//	input >> num_acts;
//
//	// initialize and read setup/precedence arrays
//	setup = new int*[num_acts];
//	precedes = new bool*[num_acts];
//	for( int i = 0; i < num_acts; i++ ) {
//		setup[i] = new int[num_acts];
//		precedes[i] = new bool[num_acts];
//		memset(precedes[i], false, sizeof(bool)*num_acts);
//	}
//
//	int sum_setup = 0;
//
//	for( int i = 0; i < num_acts; i++ ) {
//		for( int j = 0; j < num_acts; j++ ) {
//			input >> setup[i][j];
//
//			// if setup is negative, activity 'j' must precede activity 'i'
//			if( setup[i][j] == -1 ) {
//				precedes[j][i] = true;
//				setup[i][j] = 0;
//			} else {
//				sum_setup += setup[i][j];
//			}
//		}
//	}
//
//	cout << "Sum of setup times: " << sum_setup << endl;
//
//	// allocate activities
//	acts.resize(num_acts);
//	for( int i = 0; i < num_acts; i++ ) {
//		acts[i] = new Activity(env, i, 0, 90000, 0, 0, 1);
//	}
//	input.close();
//
//	cout << "Activities: " << endl;
//	for( int i = 0; i < num_acts; i++ ) {
//		cout << "\t" << acts[i]->id;
//		cout << "\tr:" << acts[i]->release;
//		cout << "\td:" << acts[i]->deadline;
//		cout << "\tp:" << acts[i]->processing;
//		//cout << "\tdue:" << acts[i]->duedate;
//		//cout << "\tw:" << acts[i]->weight;
//		cout << endl;
//	}
//}
//
//
//
//int main(int argc, char* argv[]) {
//
//	try {
//
//		// ----------------------------------------------
//		// Read input
//		// ----------------------------------------------
//
//		if ( argc != 2 ) {
//			cout << "Usage: sop [input file in TSPLIB format]" << endl;
//			exit(1);
//		}
//
//		IloEnv env;
//
//		ActivityArray acts;
//		int** setup;
//		bool** input_precedence;
//		read_sop_instance(env, argv[1], acts, setup, input_precedence);
//
//
//		// ----------------------------------------------
//		// CPO Model
//		// ----------------------------------------------
//
//		IloModel model(env);
//
//		// set of ILO activities
//
//		IloIntervalVarArray activities(env);
//		for( ActivityArray::iterator act = acts.begin(); act != acts.end(); ++act ) {
//			model.add( IloStartOf((*act)->ilo_start_time) >= (*act)->release );
//			model.add( IloEndOf((*act)->ilo_start_time) <= (*act)->deadline );
//			activities.add( (*act)->ilo_start_time );
//		}
//
//
//		// Sequence variables
//
//		IloIntArray types(env, acts.size());
//		for( unsigned int i = 0; i < acts.size(); ++i ) {
//			types[i] = i;
//		}
//		IloIntervalSequenceVar sequence(env, activities, types);
//
//
//		// Model setup times and costs (we include an additional activity to be the next
//		// of the last activity in the resource)
//
//		IloIntArray2   		  setup_costs(env, acts.size());
//		IloTransitionDistance setup_times(env, acts.size());
//		for( unsigned int i = 0; i < acts.size(); i++ ) {
//			setup_costs[i] = IloIntArray(env, acts.size()+1);
//			for( unsigned int j = 0; j < acts.size(); j++ ) {
//				setup_times.setValue(i, j, setup[i][j]);
//				setup_costs[i][j] = setup[i][j];
//			}
//			setup_costs[i][acts.size()] = 0;
//		}
//
//		// disjunctive constraint
//		model.add( IloNoOverlap(env, sequence, setup_times, IloTrue) );
//
//		// precedence constraints
//		for( unsigned int i = 0; i < acts.size(); ++i ) {
//			for( unsigned int j = i+1; j < acts.size(); ++j ) {
//				if( input_precedence[i][j] ) {
//					model.add( IloEndBeforeStart(env,
//							acts[i]->ilo_start_time,
//							acts[j]->ilo_start_time) );
//				} else if( input_precedence[j][i] ) {
//					model.add( IloEndBeforeStart(env,
//							acts[j]->ilo_start_time,
//							acts[i]->ilo_start_time) );
//				}
//			}
//		}
//
//		// objective function
//		IloIntExpr obj(env);
//		for( unsigned int i = 0; i < acts.size(); i++ ) {
//			obj += setup_costs[types[i]][IloTypeOfNext(sequence, acts[i]->ilo_start_time, acts.size())];
//		}
//		model.add( IloMinimize(env, obj) );
//
//
//
//		// ----------------------------------------------
//		// Search
//		// ----------------------------------------------
//
//		IloCP cp(model);
//
//		cp.setParameter(IloCP::Workers, 1); // only one core
//		cp.setParameter(IloCP::TimeLimit, TIME_LIMIT);
//		cp.setParameter(IloCP::DefaultInferenceLevel, IloCP::Extended);
//		cp.setParameter(IloCP::RelativeOptimalityTolerance, OPT_TOL);
//
//
//		IloIntervalSequenceVarArray seqs_array(env);
//		seqs_array.add(sequence);
//		IloSearchPhase search_phase = IloSearchPhase(env, seqs_array);
//
//		if( cp.solve( search_phase ) ) {
//			cout << "Best solution found: " << cp.getObjValue() << endl;
//		}
//
//	} catch( IloException& ex ) {
//		cout << "Error: " << ex << endl;
//	}
//
//	return 0;
//}
//
