////============================================================================
//// Main TSPTW file
//// Read and solve a TSPTW instance
////============================================================================
//
//#include <ilcp/cpext.h>
//#include <iostream>
//#include <vector>
//#include <fstream>
//#include "core/mdd_handler.hpp"
//#include "core/search_goals.hpp"
//#include "constraints/disjunctive.hpp"
//#include "util/stats.hpp"
//#include "util/util.hpp"
//#include "instances/tsptw_instances.hpp"
//
//#define TIME_LIMIT 3600
//
//// Propagation types
//#define CPO_ONLY_PROP 0
//#define MDD_ONLY_PROP 1
//#define CPO_MDD_PROP 2
//
//// Objective function
//#define MAKESPAN_OBJ 1
//#define FEASIBILITY 2
//
//// search types
//#define LEX_SEARCH 0
//#define DEFAULT_SEARCH 1
//#define MDD_SP_SEARCH 2
//
//
//using namespace std;
//
//int main_jobshop(int argc, char* argv[]) {
//
//	if( argc < 5 || ( atoi(argv[4]) > 0 && argc < 6 ) ) {
//		cout << "\nUsage: jobshop [filename] [obj-function] [search-type] [propagation-type] [width]\n" << endl;
//		cout << "\twhere: " << endl;
//		cout << "\t\tobj-function = 0: minimize sum of setup times" << endl;
//		cout << "\t\tobj-function = 1: minimize makespan" << endl;
//		cout << "\t\tobj-function = 2: no optimization, only feasibility" << endl;
//		cout << "\t\tobj-function = 3: minimize total tardiness" << endl;
//		cout << "\t\tobj-function = 4: minimize maximum lateness" << endl;
//		cout << endl;
//		cout << "\t\tsearch-type = 0: lexicographic search" << endl;
//		cout << "\t\tsearch-type = 1: default search" << endl;
//		cout << "\t\tsearch-type = 2: MDD shortest path search" << endl;
//		cout << endl;
//		cout << "\t\tpropagation-type = 0: pure CP" << endl;
//		cout << "\t\tpropagation-type = 1: pure MDD" << endl;
//		cout << "\t\tpropagation-type = 2: CP+MDD" << endl;
//		cout << endl << endl;
//
//		exit(1);
//	}
//
//	try {
//
//		IloEnv env;
//		IloModel model(env);
//
//
//		// ----------------------------------------------
//		// Read input parameters
//		// ----------------------------------------------
//
//		char inst_name[256];
//		extract_instance_name(argv[1], inst_name);
//
//		int obj_function = atoi(argv[2]);
//		int search_type = atoi(argv[3]);
//		int propagation_type = atoi(argv[4]);
//
//		int width = 0;
//
//
//		// ----------------------------------------------
//		// Initialize basic activity model
//		// ----------------------------------------------
//
//		char varname[256];
//
//		// read input file
//		ifstream input(argv[1]);
//		if( !input.is_open() ) {
//			cerr << "Error: could not open file " << argv[1] << endl;
//			exit(1);
//		}
//
//		int num_jobs, num_machines;
//		input >> num_jobs;
//		input >> num_machines;
//
//		cout << "\nInstance name: " << argv[1] << endl;
//		cout << "\tjobs: " << num_jobs << endl;
//		cout << "\tmachines: " << num_machines << endl;
//		cout << endl;
//
//		// containers for activities/intervals in each machine
//		vector<ActivityArray> act_machine(num_machines);
//		vector<IloIntervalVarArray> interv_machine(num_machines);
//		for( int i = 0; i < num_machines; ++i ) {
//			interv_machine[i] = IloIntervalVarArray(env);
//		}
//
//		// end times of all activities
//		IloIntExprArray ends(env);
//
//		int mach, proc;
//		Activity *act, *prev_act;
//		for( int j = 0; j < num_jobs; ++j ) {
//			prev_act = NULL;
//			for( int op = 0; op < num_machines; ++op ) {
//				input >> mach;
//				input >> proc;
//
//				act = new Activity(env, act_machine[mach].size(), 0, INF, proc);
//				act->ilo_start_time.setObject(act);
//
//				act_machine[mach].push_back(act);
//				interv_machine[mach].add(act->ilo_start_time);
//
//				ends.add( IloEndOf(act->ilo_start_time) );
//
//				if( prev_act != NULL ) {
//					model.add( IloEndBeforeStart(env,
//							prev_act->ilo_start_time, act->ilo_start_time) );
//				}
//				prev_act = act;
//			}
//		}
//
//		// create sequence variables
//		IloIntervalSequenceVarArray sequences(env, num_machines);
//		for( int m = 0; m < num_machines; ++m ) {
//			IloIntArray types(env, num_jobs);
//			for( int j = 0; j < num_jobs; ++j ) {
//				types[j] = j;
//			}
//			sequences[m] = IloIntervalSequenceVar(env, interv_machine[m], types);
//		}
//
//		// makespan variable
//		IloIntVar makespan(env, 0, INF);
//		model.add( makespan == IloMax(ends) );
//
//
//		// ----------------------------------------------
//		// Create CPO model
//		// ----------------------------------------------
//
//		if( propagation_type == CPO_ONLY_PROP || propagation_type == CPO_MDD_PROP ) {
//			for( int m = 0; m < num_machines; ++m ) {
//				model.add( IloNoOverlap(env, sequences[m]) );
//			}
//		}
//
//
//		// ----------------------------------------------
//		// Create MDD model
//		// ----------------------------------------------
//
//		vector<IloIntVarArray> permutations(num_machines);
//
//		if( propagation_type == MDD_ONLY_PROP || propagation_type == CPO_MDD_PROP ) {
//
//			// Read width
//			width = atoi(argv[5]);
//			if( width <= 0 ) {
//				cout << "\nError: invalid width" << endl << endl;
//				exit(1);
//			}
//			cout << "width: " << width << endl << endl;
//
//			// dummy setup_times array
//			int** setup = new int*[num_jobs];
//			for( int j = 0; j < num_jobs; ++j ) {
//				setup[j] = new int[num_jobs];
//				memset(setup[j], 0, sizeof(int)*num_jobs);
//			}
//
//			// dummy setup times / total tardiness
//			IloIntVar setup_times(env, 0, 1);
//			IloIntVar total_tardiness(env, 0, 1);
//			model.add( setup_times == 0 );
//			model.add( total_tardiness == 0 );
//
//			for( int m = 0; m < num_machines; ++m ) {
//
//				// Allocate permutation variables
//				permutations[m] = IloIntVarArray(env, num_jobs);
//			    for( int j = 0; j < num_jobs; ++j ) {
//			      sprintf(varname, "permutation[%d][%d]", m, j);
//			      permutations[m][j] = IloIntVar(env, 0, num_jobs-1, varname);
//			    }
//			    model.add(IloAllDiff(env, permutations[m]));
//
//			    // Create machine makespan
//			    sprintf(varname, "machine_makespan[%d]", m);
//			    IloIntVar machine_makespan(env, 0, INF, varname);
//			    IloIntExprArray ends_machine(env);
//			    for( int j = 0; j < num_jobs; ++j ) {
//			    	ends_machine.add( IloEndOf(interv_machine[m][j]) );
//			    }
//			    model.add( machine_makespan == IloMax(ends_machine) );
//			    model.add( machine_makespan <= makespan );
//
//
//				// create MDD disjunctive propagator
//				IloArray<MDDPropagator*> propagators(env);
//				DisjunctivePropagator* disj = new DisjunctivePropagator(act_machine[m],
//				                                            sequences[m], permutations[m],
//				                                            setup, NULL,
//				                                            setup_times, makespan,
//				                                            total_tardiness);
//				propagators.add( disj );
//
//				// create MDD handler with propagators and add it to model
//				model.add( IloMDDHandler(env, permutations[m], interv_machine[m],
//						sequences[m], propagators, width, "mdd-handler") );
//			}
//		}
//
//
//
//		// ---------------------------------------
//		// Objective function
//		// ---------------------------------------
//
//		switch( obj_function ) {
//		case MAKESPAN_OBJ:
//			// minimize makespan
//			model.add( IloMinimize(env, makespan) );
//			break;
//		case FEASIBILITY:
//			// feasibility only
//			break;
//		default:
//			cout << "Error: objective function undefined." << endl;
//			exit(1);
//		}
//
//
//		// ----------------------------------------------
//		// Search parameters
//		// ----------------------------------------------
//
//		IloCP cp(model);
//
//		// set parameters
//		cp.setParameter(IloCP::Workers, 1); // only one core
//		cp.setParameter(IloCP::TimeLimit, TIME_LIMIT);
//		cp.setParameter(IloCP::DefaultInferenceLevel, IloCP::Extended);
//        if( search_type != DEFAULT_SEARCH ) {
//          cp.setParameter(IloCP::SearchType, IloCP::DepthFirst);
//        }
//        //cp.setParameter(IloCP::LogVerbosity, IloCP::Quiet);
//
//
//
//        // ----------------------------------------------
//		// Search strategies
//		// ----------------------------------------------
//
//		// set search strategy
//        if( search_type == LEX_SEARCH ) {
//
//          // create search method
//          IloGoal search_procedure;
//          if( propagation_type == MDD_ONLY_PROP ) {
//
//        	  IloIntVarArray all_permutations(env);
//        	  for( int m = 0; m < num_machines; ++m ) {
//        		  all_permutations.add(permutations[m]);
//        	  }
//
//        	  search_procedure = IloGenerate(env, all_permutations);
//        	  search_procedure = search_procedure && IloFixIntVarMin(env, makespan);
//        	  for( int m = 0; m < num_machines; ++m ) {
//        		  search_procedure = search_procedure
//        				  && IloSetMinStartTime(env, interv_machine[m]);
//        	  }
//
//          } else {
//
//        	  // CP or CP+MDD: lexicographic search on sequence of activities
//        	  search_procedure = IloGoalTrue(env);
//        	  for( int m = 0; m < num_machines; ++m ) {
//        		  search_procedure = search_procedure
//        				  && IloLexSchedule(env, interv_machine[m], sequences[m]);
//        	  }
//        	  search_procedure = search_procedure && IloFixIntVarMin(env, makespan);
//        	  for( int m = 0; m < num_machines; ++m ) {
//        		  search_procedure = search_procedure
//        				  && IloSetMinStartTime(env, interv_machine[m]);
//        	  }
//          }
//          cp.startNewSearch(search_procedure);
//
//        } else if( search_type == DEFAULT_SEARCH ) {
//
//          IloSearchPhase search_phase;
//          if( propagation_type == 1 ) {
//
//        	  IloIntVarArray all_permutations(env);
//        	  for( int m = 0; m < num_machines; ++m ) {
//        		  all_permutations.add(permutations[m]);
//        	  }
//        	  search_phase = IloSearchPhase(env, all_permutations);
//
//          } else {
//        	  search_phase = IloSearchPhase(env, sequences);
//          }
//          cp.startNewSearch(search_phase);
//
//        } else if( search_type == MDD_SP_SEARCH ) {
//
//          if( propagation_type == 0 ) {
//            cout << "Error: Propagation type must include MDD" << endl;
//            exit(1);
//          }
//
////          IloGoal search_procedure = IloShortestPathMDD(env, permutation, disj_propagator)
////                && IloSetTimesPermutation(env, permutation, activities, sum_setup, makespan)
////                && IloFixIntVarMin(env, total_tardiness);
////
////          cp.startNewSearch(search_procedure);
//        }
//
//        // ----------------------------------------------
//		// Search
//		// ----------------------------------------------
//
//        bool has_sol = false;
//		while( cp.next() ) {
//			has_sol = true;
//		}
//
//		if( has_sol ) {
//
//			if( propagation_type == MDD_ONLY_PROP ) {
//				cout << "\n";
//				for( int m = 0; m < num_machines; ++m ) {
//					cout << "Machine " << m << endl;
//					//IlcIntervalSequenceVar ilcseq = cp.getIntervalSequence(sequences[m]);
//					for( int op = 0; op < num_jobs; ++op ) {
//						cout << "\t";
//						//cout << "Activity " << ilcseq.getType(cp.getInterval(act));
//						cout << "Job " << act_machine[m][cp.getValue(permutations[m][op])]->id;
//						cout << " :: " << cp.getInterval(interv_machine[m][cp.getValue(permutations[m][op])]);
//						cout << endl;
//					}
//					cout << endl;
//				}
//			} else {
//				cout << "\n";
//				for( int m = 0; m < num_machines; ++m ) {
//					cout << "Machine " << m << endl;
//					//IlcIntervalSequenceVar ilcseq = cp.getIntervalSequence(sequences[m]);
//					for( IloIntervalVar act = cp.getFirst(sequences[m]);
//							act.getImpl() != NULL;
//							act = cp.getNext(sequences[m], act) )
//					{
//						cout << "\t";
//						//cout << "Activity " << ilcseq.getType(cp.getInterval(act));
//						cout << "Job " << (static_cast<Activity*>(act.getObject()))->id;
//						cout << " :: " << cp.getInterval(act);
//						cout << endl;
//					}
//					cout << endl;
//				}
//			}
//			cout << "\nFinal Makespan: " << cp.getIntVar(makespan) << endl;
//		}
//
//		cout << "\nBranches: " << cp.getInfo(IloCP::NumberOfBranches) << endl;
//		cout << "Fails: " << cp.getInfo(IloCP::NumberOfFails) << endl;
//		cout << "Time: " << cp.getTime() << endl << endl;
//
//
//
//	} catch (IloException &ex) {
//		cout << "CPO Error: " << ex << endl;
//	}
//
//	return 0;
//}
//
//
