////============================================================================
//// Sequential Ordering Problems
//// Main file
////============================================================================
//
//#include <ilcp/cpext.h>
//#include <iostream>
//#include <vector>
//#include <fstream>
//
//#include "core/mdd_handler.hpp"
//#include "core/search_goals.hpp"
//#include "constraints/disjunctive.hpp"
//#include "util/stats.hpp"
//#include "util/util.hpp"
//
//#include "instances/tsptw_instances.hpp"
//
//
//using namespace std;
//
//
//
///**
// * Main
// */
//int main_sop(int argc, char* argv[]) {
//
//	if( argc < 5 || ( atoi(argv[4]) > 0 && argc < 6 ) ) {
//		cout << "\nUsage: disjunctive_sop [filename] [input-type] [obj-function] [propagation-type] [width]\n" << endl;
//		cout << "\twhere: " << endl;
//		cout << "\t\tinput_type = 0: TSP format" << endl;
//		cout << endl;
//		cout << "\t\tobj-function = 0: minimize sum of setup times" << endl;
//		cout << "\t\tobj-function = 1: no optimization, only feasibility" << endl;
//		cout << endl;
//		cout << "\t\tpropagation-type = 0: pure CP" << endl;
//		cout << "\t\tpropagation-type = 1: pure MDD" << endl;
//		cout << "\t\tpropagation-type = 2: CP+MDD" << endl;
//		cout << endl << endl;
//
//		exit(1);
//	}
//
//
//	try {
//
//		IloEnv env;
//		IloModel model(env);
//
//		// ----------------------------------------------
//		// Read input
//		// ----------------------------------------------
//
//		int input_type = atoi(argv[2]);
//		int obj_function = atoi(argv[3]);
//		int propagation_type = atoi(argv[4]);
//		int width = -1;
//
//		ActivityArray acts;
//		int** setup;
//		bool** precedes;
//
//		switch( input_type ) {
//		case 0:
//			read_sop_input(env, argv[1], acts, setup, precedes);
//			break;
//		default:
//			cout << "Error: instance type cannot be identified." << endl;
//			exit(1);
//		}
//
//		cout << "\nInstance name: " << argv[1] << endl;
//
//
//		// -----------------------------------------------
//		// ILOG CPO Model
//		// -----------------------------------------------
//
//		// set of ILO activities
//		IloIntervalVarArray activities(env);
//
//
//		// collect activities
//		for( ActivityArray::iterator act = acts.begin(); act != acts.end(); ++act ) {
//			// add to set of activities
//			activities.add((*act)->ilo_start_time);
//		}
//
//
//		/*
//		 * Permutation variables
//		 */
//		IloIntVarArray permutation;
//
//		if( propagation_type == 1 || propagation_type == 2 ) {
//
//			permutation = IloIntVarArray(env, acts.size(), 0, acts.size()-1);
//
//			// first city is initial depot
//			permutation[0].setMax(0);
//
//			// last city is terminal depot
//			permutation[acts.size()-1].setMin(acts.size()-1);
//
//			// Enforce that all permutation variables must be distinct
//			// Note: this is added outside the disjunctive constraint to allow the alldiff CPO propagator
//			// to be performed first
//			model.add( IloAllDiff(env, permutation) );
//		}
//
//		// sum of setup times and makespan variable
//		IloIntVar sum_setup(env, 0, INF);
//		IloIntVar makespan(env, 0, INF);
//		model.add( sum_setup >= 0 );	// force CPO extraction
//		model.add( makespan >= 0 );
//
//
//		// create sequence variable with empty handle; it is only initialized for
//		// CP and CP+MDD technique
//		IloIntervalSequenceVar sequence;
//
//		// ------------------------------------------------------
//		// ILOG CP disjunctive propagator
//		// ------------------------------------------------------
//
//		if( propagation_type == 0 || propagation_type == 2 ) {
//
//			// define activity times
//			IloIntArray types(env, acts.size());
//			for( unsigned int i = 0; i < acts.size(); i++ ) {
//				types[i] = i;
//			}
//
//			// create sequence variables
//			sequence = IloIntervalSequenceVar(env, activities, types);
//
//			// we fix first and last activity in the sequence
//			model.add( IloFirst(env, sequence, acts[0]->ilo_start_time) );
//			model.add( IloLast(env, sequence, acts.back()->ilo_start_time) );
//
//			// create transition cost matrix
//			IloIntArray2   		  setup_costs(env, acts.size());
//			IloTransitionDistance setup_times(env, acts.size());
//
//			// enforce precedence relations
//			for( unsigned int i = 0; i < acts.size() && i < 5; i++ ) {
//				for( unsigned int j = 0; j < acts.size() && j < 2; j++ ) {
//					if( precedes[i][j] ) {
//						//cout << i << " << " << j << endl;
//						//model.add( IloEndOf(acts[i]->ilo_start_time) <= IloStartOf(acts[j]->ilo_start_time) );
//					}
//				}
//			}
//
//			// Type of next for the last activity on the machine
//			for( unsigned int i = 0; i < acts.size(); i++ ) {
//				setup_costs[i] = IloIntArray(env, acts.size());
//				for( unsigned int j = 0; j < acts.size(); j++ ) {
//					// ignore negative setup times: they are just indicating precedence relations
//					if( setup[i][j] < 0 ) {
//						setup_times.setValue(i, j, 0);
//						setup_costs[i][j] = 0;
//					} else {
//						setup_times.setValue(i, j, setup[i][j]);
//						setup_costs[i][j] = setup[i][j];
//					}
//				}
//			}
//
//			// add CP optimizer disjunctive constraint
//			model.add( IloNoOverlap(env, sequence, setup_times, IloTrue) );
//
//			// link time variables with objective function variables. We do not add
//			// setup times linking for minimization of makespan, since it necessarily
//			// includes a set of element constraints that may greatly slow down the solving process
//			if( obj_function == 0 ) {
//				IloIntExpr obj(env);
//				for( int i = 0; i < (int)acts.size(); i++ ) {
//					obj += setup_costs[types[i]][IloTypeOfNext(sequence, acts[i]->ilo_start_time, 0)];
//				}
//				model.add( sum_setup == obj);
//
//			} else if( propagation_type == 0 ) {
//				sum_setup.setMax(0);
//			}
//		}
//
//
//
//		// ------------------------------------------------------
//		// MDD Propagator
//		// ------------------------------------------------------
//
//		if( propagation_type == 1 || propagation_type == 2 ) {
//
//			// read width
//			width = atoi(argv[5]);
//			if( width <= 0 ) {
//				cout << "\nError: invalid width" << endl << endl;
//				exit(1);
//			}
//			cout << "width: " << width << endl << endl;
//
//			// create MDD disjunctive propagator
//			IloArray<MDDPropagator*> propagators(env);
//			//propagators.add( new DisjunctivePropagator(acts, sequence, permutation, setup, precedes, sum_setup, makespan) );
//
//			// create MDD handler with propagators and add it to model
//			model.add( IloMDDHandler(env, permutation, activities, sequence, propagators, width, "mdd-handler") );
//
//		}
//
//
//		// ---------------------------------------
//		// Objective function
//		// ---------------------------------------
//
//		switch( obj_function ) {
//		case 0:
//			// minimize sum of setup times
//			model.add( IloMinimize(env, sum_setup) );
//			break;
//		case 1:
//			// feasibility only
//			break;
//		default:
//			cout << "Error: objective function undefined." << endl;
//			exit(1);
//		}
//
//
//		// -----------------------------------------------
//		// Solve
//		// -----------------------------------------------
//
//		// solve problem with lexicographic search
//		IloCP cp(model);
//		cp.setParameter(IloCP::Workers, 1); // only one core
//		cp.setParameter(IloCP::TimeLimit, 3600);
//		cp.setParameter(IloCP::SearchType, IloCP::DepthFirst);
//		cp.setParameter(IloCP::DefaultInferenceLevel, IloCP::Extended);
//        //cp.setParameter(IloCP::LogVerbosity, IloCP::Quiet);
//
//		// create search method
//		IloGoal search_procedure;
//		if( propagation_type == 1 ) {
//
//			// Pure MDD: perform lexicographic search on permutation variables
//			search_procedure = IloGenerate(env, permutation)
//					&& IloSetTimesPermutation(env, permutation, activities, sum_setup, makespan);
//
//		} else {
//
//			// CP or CP+MDD: lexicographic search on sequence of activities
//			search_procedure = IloSetPrecedences(env, activities, sequence, precedes)
//					&& IloLexSchedule(env, activities, sequence)
//					&& IloSetTimesSequence(env, sequence, sum_setup, makespan);
//
//		}
//
//
////		// generate all solutions
////		cp.setParameter(IloCP::LogVerbosity, IloCP::Quiet);
////		long int sol_count = 0;
////		cp.startNewSearch(search_procedure);
////		while( cp.next() ) {
////			if( propagation_type == 1 ) {
////				for( int i = 0; i < permutation.getSize(); i++ ) {
////					cout << cp.getValue(permutation[i]) << " ";
////				}
////				cout << endl;
////			} else {
////				IlcIntervalSequenceVar ilcseq = cp.getIntervalSequence(sequence);
////				for( IloIntervalVar act = cp.getFirst(sequence); act.getImpl() != NULL; act = cp.getNext(sequence, act) ) {
////					cout << ilcseq.getType(cp.getInterval(act)) << " ";
////				}
////				cout << endl;
////			}
////
////			if( ++sol_count == 1e10 )
////				break;
////		}
////		exit(1);
//
//
////		if( cp.propagate() ) {
////			cout << cp.getIntVar(makespan) << endl;
////			for( int i = 0; i < permutation.getSize(); i++ ) {
////				cout << " " << cp.getIntVar(permutation[i]);
////			}
////		} else {
////			cout << endl << endl << "Infeasible!" << endl << endl;
////		}
////		cout << endl;
////		exit(1);
//
//		//IloIntervalSequenceVarArray seqs_array(env);
//		//seqs_array.add(sequence);
//		//if( cp.solve(IloSearchPhase(env, seqs_array)) ) {
//
//		if( cp.solve(search_procedure) ) {
//		//if( cp.solve() ) {
//
//			// printing
//			if( propagation_type == 1 ) {
//
//				cout << "\nSequence: " << endl;
//				for( int i = 0; i < permutation.getSize(); i++ ) {
//					//cout << cp.getValue(permutation[i]) << " ";
//					cout << "\t";
//					cout << "Activity " << cp.getValue(permutation[i]);
//					cout << " :: " << cp.getInterval(activities[cp.getValue(permutation[i])]);
//					cout << endl;
//				}
//				cout << endl;
//
//			} else {
//
//				cout << "\nSequence: " << endl;
//				IlcIntervalSequenceVar ilcseq = cp.getIntervalSequence(sequence);
//				int i = 0;
//				for( IloIntervalVar act = cp.getFirst(sequence); act.getImpl() != NULL; act = cp.getNext(sequence, act) ) {
//					cout << "\t";
//					cout << "Activity " << ilcseq.getType(cp.getInterval(act));
//					if( permutation.getImpl() != NULL ) {
//						cout << " (" << cp.getValue(permutation[i++]) << ")";
//					}
//					cout << " :: " << cp.getInterval(act);
//					cout << endl;
//				}
//				cout << endl;
//
//			}
//
//			cout << "Makespan: " << cp.getIntVar(makespan) << endl;
//			cout << "Sum setup times: " << cp.getIntVar(sum_setup) << endl;
//			cout << endl;
//		}
//
//		cout << "\nBranches: " << cp.getInfo(IloCP::NumberOfBranches) << endl;
//		cout << "Fails: " << cp.getInfo(IloCP::NumberOfFails) << endl;
//		cout << "Time: " << cp.getTime() << endl << endl;
//
//
//		// statistics
//		ofstream statsfile("stats.txt", ios::app);
//		statsfile << argv[1];
//
//		switch( propagation_type ) {
//		case 0:
//			statsfile << "\tCP";
//			break;
//		case 1:
//			statsfile << "\tMDD";
//			break;
//		case 2:
//			statsfile << "\tCP+MDD";
//			break;
//		}
//
//		if( propagation_type == 0 ) {
//			statsfile << "\tnowidth";
//		} else {
//			statsfile << "\t" << width;
//		}
//
//		statsfile << "\t" << cp.getIntVar(sum_setup);
//		statsfile << "\t" << cp.getIntVar(makespan);
//		statsfile << "\t" << cp.getInfo(IloCP::NumberOfBranches);
//		statsfile << "\t" << cp.getInfo(IloCP::NumberOfFails);
//		statsfile << "\t" << cp.getTime();
//		statsfile << endl;
//		statsfile.close();
//
//	} catch (IloException &ex) {
//		cout << "CPO Error: " << ex << endl;
//	}
//
//	return 0;
//}
//
//
//
//
//
//
